package pps.core.base.utils;

import cn.hutool.core.collection.CollUtil;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.SystemAreaService;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.sys_area.QuerySysAreaInput;
import pps.cloud.system.service.data.sys_area.QuerySysAreaOutput;
import pps.cloud.system.service.data.sys_dictionary.GetSysDictionaryViewInput;
import pps.cloud.system.service.data.sys_dictionary.GetSysDictionaryViewOutput;
import pps.cloud.system.service.data.sys_dictionary.QuerySysDictionaryViewInput;
import pps.cloud.system.service.data.sys_dictionary.QuerySysDictionaryViewOutput;
import pps.cloud.system.service.data.sys_organization.GetAllOuListByOuIdInput;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewInput;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewOutput;
import xstartup.base.XContext;
import xstartup.data.XListResult;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 常用服务调用工具类
 *
 * @author ZWT
 * @date 2023/08/25 16:08
 */
public class ServiceUtil {

    /*-------------------------服务调用-------------------------*/

    /**
     * 通过字典类型查询字典
     *
     * @param context 上下文
     * @param alias   别名
     * @return {@link Map}<{@link String}, {@link String}>
     */
    public static Map<String, String> getDictMap(XContext context, String alias) {
        QuerySysDictionaryViewInput dictInput = new QuerySysDictionaryViewInput();
        dictInput.setAlias(alias);
        dictInput.setPage(1);
        dictInput.setLimit(999);
        SystemDictionaryService dictionaryService = context.getBean(SystemDictionaryService.class);
        XListResult<QuerySysDictionaryViewOutput> dictPageResult = dictionaryService.queryChildSysDictionarysByParentAlias(context, dictInput);
        dictPageResult.throwIfFail();
        return dictPageResult.getResult().stream()
                .collect(Collectors.toMap(QuerySysDictionaryViewOutput::getDicKey, QuerySysDictionaryViewOutput::getDicName));
    }

    /**
     * 获取所有path包含组织ID的组织
     *
     * @param context 上下文
     * @param ouId    ou id
     * @return {@link List}<{@link String}>
     */
    public static List<String> getOrgIdsByPath(XContext context, String ouId) {
        SysOrganizationCloudService organizationCloudService = context.getBean(SysOrganizationCloudService.class);
        GetAllOuListByOuIdInput ouIdInput = new GetAllOuListByOuIdInput();
        ouIdInput.setOuId(ouId);
        XListResult<GetSysOrganizationViewOutput> allListByOuId = organizationCloudService.getAllListByOuId(context, ouIdInput);
        allListByOuId.throwIfFail();
        return allListByOuId.getResult().stream()
                .map(GetSysOrganizationViewOutput::getId)
                .collect(Collectors.toList());
    }

    /**
     * 获取省市列表
     *
     * @param context  上下文
     * @param province 省份
     * @return {@link List }<{@link QuerySysAreaOutput }>
     */
    public static List<QuerySysAreaOutput> getProvinceAndCityList(XContext context, String... province) {
        SystemAreaService bean = context.getBean(SystemAreaService.class);
        XListResult<QuerySysAreaOutput> result = bean.getProvinceAndCity(context, QuerySysAreaInput.builder()
                .provinceList(CollUtil.toList(province))
                .build());
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取采油厂列表
     *
     * @param context 上下文
     * @return {@link List }<{@link GetSysOrganizationViewOutput }>
     */
    public static List<GetSysOrganizationViewOutput> getPlantList(XContext context) {
        SysOrganizationCloudService bean = context.getBean(SysOrganizationCloudService.class);
        XListResult<GetSysOrganizationViewOutput> result = bean.getPlantList(context, GetSysOrganizationViewInput.builder().build());
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取字典列表
     *
     * @param context 上下文
     * @param alias   别名
     * @return {@link List }<{@link GetSysDictionaryViewOutput }>
     */
    public static List<GetSysDictionaryViewOutput> getDictionaryList(XContext context, String alias) {
        SystemDictionaryService bean = context.getBean(SystemDictionaryService.class);
        XListResult<GetSysDictionaryViewOutput> result = bean.querySysDictionaryByParam(context, GetSysDictionaryViewInput.builder()
                .alias(alias)
                .build());
        result.throwIfFail();
        return result.getResult();
    }
}