package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.page.PageMethod;
import pps.cloud.base.service.BasePhotovoltaicPlantCloudService;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudInput;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudOutput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailInput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailOutput;
import pps.cloud.prediction.service.IThirdPowerCloudService;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerInput;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerOutput;
import pps.cloud.prediction.service.data.third_daily_accumulation_update.GetThirdDailyAccumulationUpdateInput;
import pps.cloud.prediction.service.data.third_daily_accumulation_update.GetThirdDailyAccumulationUpdateOutput;
import pps.cloud.prediction.service.data.wellhead_daily_production_situation.GetWellheadDailyProductionSituationInput;
import pps.cloud.prediction.service.data.wellhead_daily_production_situation.GetWellheadDailyProductionSituationOutput;
import pps.cloud.space.service.IDailyElectricityTrendCloudService;
import pps.cloud.space.service.data.line_daily_electricity_trend.GetLineDailyElectricityTrendInput;
import pps.cloud.space.service.data.line_daily_electricity_trend.GetLineDailyElectricityTrendOutput;
import pps.cloud.space.service.data.line_daily_electricity_trend.GetValleyElectricRateOutput;
import pps.cloud.space.service.data.well_daily_electricity_trend.GetWellDailyAverageInput;
import pps.cloud.space.service.data.well_daily_electricity_trend.GetWellDailyElectricityTrendInput;
import pps.cloud.space.service.data.well_daily_electricity_trend.GetWellDailyElectricityTrendOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.*;
import pps.core.space.mapper.*;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.helper.XTransactionHelper;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 日用电趋势计算Cloud模块
 *
 * @author ZWT
 * @date 2024/05/22 17:17
 */
@XService
public class DailyElectricityTrendCloudServiceImpl implements IDailyElectricityTrendCloudService {

    /**
     * 日用电趋势计算
     * <p>
     * (1)取昨日极短期间开优化结果
     * (2)绿电阶段开井时间：每15分钟判段光伏出力>运行功率（考虑开井顺序,功率累加） 并且当前时段开井，则算坐开井时段(0.25小时)，全天累加
     * (3)单井绿电用电：运行功率*绿电阶段开井时间
     * (4)市电用电:总用电-光伏用电
     *
     * @param context 上下文
     * @return {@link XServiceResult }
     */
    @Override
    public XServiceResult dailyElectricityTrendJob(XContext context) {
        DateTime yesterday = DateUtil.beginOfDay(DateUtil.yesterday());
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        //查每15分钟的有功功率
        Map<String, List<GetThirdActivePowerOutput>> powerMap = this.getAvgPhotovoltaicPower(context, yesterday, oilFieldCode);
        //查昨日间开优化结果
        Map<String, List<SpaceOptimizeShortDurationView>> lineMap = this.getOptimizationResult(context, yesterday);
        //查昨日井口生产情况
        Map<String, GetWellheadDailyProductionSituationOutput> wellProductionMap = this.getWellProductionList(context, yesterday, oilFieldCode);
        //查昨日日累计用电
        Map<String, BigDecimal> dailyUpdatePowerMap = this.getDailyUpdatePowerMap(context, yesterday, oilFieldCode);
        //查去年同期用电趋势
        Map<String, LineDailyElectricityTrendEnt> lastElectricityTrendMap = this.getLastElectricityTrendMap(context, yesterday, oilFieldCode);
        //用电趋势计算结果
        List<LineDailyElectricityTrendView> stationSaveList = new ArrayList<>(powerMap.size());
        //井口用电趋势计算结果
        List<WellDailyElectricityTrendView> wellSaveList = new ArrayList<>(wellProductionMap.size());
        //峰谷策略ID/峰谷策略LIST Map
        Map<String, List<GetBasePriceStrategyDetailOutput>> strategyMap = ServiceUtil.getStrategyDetailList(context, GetBasePriceStrategyDetailInput.builder()
                        .strategyMonth(String.valueOf(yesterday.monthBaseOne()))
                        .periodTypeKey("LOW")
                        .build()).stream()
                .collect(Collectors.groupingBy(GetBasePriceStrategyDetailOutput::getStrategyId));
        //电站名/市电峰谷ID Map
        BasePhotovoltaicPlantCloudService plantCloudService = context.getBean(BasePhotovoltaicPlantCloudService.class);
        XListResult<GetBasePhotovoltaicPlantCloudOutput> placeListResult = plantCloudService.getPlantStrategyList(context, GetBasePhotovoltaicPlantCloudInput.builder().build());
        placeListResult.throwIfFail();
        Map<String, String> plantStrategyMap = placeListResult.getResult().stream()
                .collect(Collectors.toMap(GetBasePhotovoltaicPlantCloudOutput::getStationName, GetBasePhotovoltaicPlantCloudOutput::getStrategyId));
        //获取当前配置
        SpaceOptimizeShortDurationView wellInfo;
        BigDecimal currentServiceRating;
        String wellNumber;
        List<GetThirdActivePowerOutput> thirdActivePowerList;
        Map<Integer, List<SpaceOptimizeShortDurationView>> startSeqMap;
        List<SpaceOptimizeShortDurationView> durationList;
        GetThirdActivePowerOutput thirdActivePower;
        BigDecimal dailyElectricityConsumption;
        BigDecimal photovoltaicPower;
        BigDecimal serviceRating;
        BigDecimal sumValleyElectricOpenHour;
        BigDecimal sumRunDuration;
        BigDecimal dailyGreenOpenHour;
        BigDecimal valleyElectricOpenHour;
        List<GetBasePriceStrategyDetailOutput> strategyList;
        //遍历全量电站
        for (String stationName : powerMap.keySet()) {
            //取存在的间开
            if (lineMap.containsKey(stationName)) {
                //每15分钟有功功率
                thirdActivePowerList = powerMap.get(stationName);
                //按照启动顺序分组
                startSeqMap = lineMap.get(stationName).stream()
                        .collect(Collectors.groupingBy(SpaceOptimizeShortDurationView::getStartSeq));
                //光伏日累计用电(绿电)
                photovoltaicPower = BigDecimal.ZERO;
                //按启动顺序取时间段列表
                serviceRating = BigDecimal.ZERO;
                //取谷电时间
                strategyList = strategyMap.get(plantStrategyMap.get(stationName));
                //谷电总开井时长
                sumValleyElectricOpenHour = BigDecimal.ZERO;
                //总运行时长时长
                sumRunDuration = BigDecimal.ZERO;
                for (int i = 1; i <= startSeqMap.size(); i++) {
                    durationList = startSeqMap.get(i);
                    //当前井口
                    wellInfo = durationList.get(0);
                    //当前井运行功率
                    currentServiceRating = wellInfo.getServiceRating();
                    //井号
                    wellNumber = wellInfo.getWellNumber();
                    //累加运行功率
                    serviceRating = serviceRating.add(currentServiceRating);
                    //绿电开井时间(h)
                    dailyGreenOpenHour = BigDecimal.ZERO;
                    //谷电开井时长(h)
                    valleyElectricOpenHour = BigDecimal.ZERO;
                    //记录每一次计算结束位置
                    int index = 0;
                    for (SpaceOptimizeShortDurationView duration : durationList) {
                        //时间转换
                        DateTime openWellTime = DateUtil.parse(duration.getOpenWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                        DateTime closeWellTime;
                        if (CharSequenceUtil.equals(duration.getCloseWellTime(), BusinessConstant.END_OF_DAY_TIME)) {
                            closeWellTime = BusinessConstant.DATE_FLAG;
                        } else {
                            closeWellTime = DateUtil.parse(duration.getCloseWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                        }
                        //遍历功率
                        for (int i1 = index; i1 < thirdActivePowerList.size(); i1++) {
                            thirdActivePower = thirdActivePowerList.get(i1);
                            //在时间范围内,并且总运行功率满足运行
                            if (DateUtil.isIn(thirdActivePower.getCreateDate(), openWellTime, closeWellTime) && thirdActivePower.getPhotovoltaicPower().compareTo(serviceRating) >= 0) {
                                //当前井口绿电运行时长(累加0.25小时)
                                dailyGreenOpenHour = dailyGreenOpenHour.add(BusinessConstant.BIG_DECIMAL_0_25);
                            }
                        }
                        //谷电时长计算
                        if (CollUtil.isNotEmpty(strategyList)) {
                            for (GetBasePriceStrategyDetailOutput strategy : strategyList) {
                                //时间转换
                                DateTime startTime = DateUtil.parse(strategy.getStartTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                                if (DateUtil.compare(startTime, closeWellTime) >= 0) {
                                    continue;
                                }
                                DateTime endTime;
                                if (CharSequenceUtil.equals(strategy.getEndTime(), BusinessConstant.END_OF_DAY_TIME)) {
                                    endTime = BusinessConstant.DATE_FLAG;
                                } else {
                                    endTime = DateUtil.parse(strategy.getEndTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                                }
                                if (DateUtil.compare(endTime, openWellTime) <= 0) {
                                    continue;
                                }
                                //计算谷电时长
                                if (DateUtil.compare(openWellTime, startTime) > 0) {
                                    startTime = openWellTime;
                                }
                                if (DateUtil.compare(closeWellTime, endTime) < 0) {
                                    endTime = closeWellTime;
                                }
                                valleyElectricOpenHour = BigDecimal.valueOf(DateUtil.between(startTime, endTime, DateUnit.MINUTE)).divide(BusinessConstant.BIG_DECIMAL_60, 2, RoundingMode.HALF_UP);
                            }
                        }
                    }
                    //单井绿电用电：运行功率*绿电阶段开井时间
                    BigDecimal dailyGreenConsumption = dailyGreenOpenHour.multiply(currentServiceRating);
                    //井场绿电用电累加
                    photovoltaicPower = photovoltaicPower.add(dailyGreenConsumption);
                    //用电趋势结果计算
                    wellSaveList.add(this.getWellDailyElectricityTrendView(wellProductionMap.get(wellNumber), wellNumber,
                            wellInfo.getRunDuration(), dailyGreenOpenHour, dailyGreenConsumption, yesterday, valleyElectricOpenHour, oilFieldCode));
                    //累加总运行时长
                    sumRunDuration = sumRunDuration.add(wellInfo.getRunDuration());
                    //累加谷电运行时长
                    sumValleyElectricOpenHour = sumValleyElectricOpenHour.add(valleyElectricOpenHour);
                }
                //计算封装井场用电趋势数据
                dailyElectricityConsumption = Optional.ofNullable(dailyUpdatePowerMap.get(stationName)).orElse(BigDecimal.ZERO);
                stationSaveList.add(
                        this.getLineDailyElectricityTrendView(lastElectricityTrendMap.get(stationName),
                                dailyElectricityConsumption, photovoltaicPower, stationName, yesterday, sumRunDuration, sumValleyElectricOpenHour, oilFieldCode)
                );
            }
        }
        //开启事务批量插入
        return XTransactionHelper.begin(context, () -> {
            int size;
            if (CollUtil.isNotEmpty(stationSaveList)) {
                LineDailyElectricityTrendViewMapper mapper = context.getBean(LineDailyElectricityTrendViewMapper.class);
                size = stationSaveList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<LineDailyElectricityTrendView>> subList = BaseUtils.getSubList(stationSaveList);
                    subList.forEach(mapper::batchInsert);
                } else {
                    mapper.batchInsert(stationSaveList);
                }
            }
            if (CollUtil.isNotEmpty(wellSaveList)) {
                WellDailyElectricityTrendViewMapper mapper = context.getBean(WellDailyElectricityTrendViewMapper.class);
                size = wellSaveList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<WellDailyElectricityTrendView>> subList = BaseUtils.getSubList(wellSaveList);
                    subList.forEach(mapper::batchInsert);
                } else {
                    mapper.batchInsert(wellSaveList);
                }
            }
            return XServiceResult.OK;
        });
    }

    /**
     * 用电趋势统计
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetLineDailyElectricityTrendOutput }>
     */
    @Override
    public XListResult<GetLineDailyElectricityTrendOutput> queryDailyElectricityTrend(XContext context, GetLineDailyElectricityTrendInput input) {
        LineDailyElectricityTrendViewMapper mapper = context.getBean(LineDailyElectricityTrendViewMapper.class);
        List<LineDailyElectricityTrendView> list = mapper.selectDailyElectricityTrend(
                LineDailyElectricityTrendView.builder()
                        .startTime(input.getStartTime())
                        .endTime(input.getEndTime())
                        .stationNameList(input.getStationNameList())
                        .systemSource(input.getSystemSource())
                        .build()
        );
        return XListResult.success(XCopyUtils.copyNewList(list, GetLineDailyElectricityTrendOutput.class));
    }

    /**
     * 累积用电
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetLineDailyElectricityTrendOutput }>
     */
    @Override
    public XSingleResult<GetLineDailyElectricityTrendOutput> querySumDailyElectricity(XContext context, GetLineDailyElectricityTrendInput input) {
        LineDailyElectricityTrendMapper mapper = context.getBean(LineDailyElectricityTrendMapper.class);
        LineDailyElectricityTrendEnt ent = mapper.selectOne(new QueryWrapper<LineDailyElectricityTrendEnt>()
                .select("IFNULL( SUM( valley_electric_open_hour ), 0 ) AS valley_electric_open_hour",
                        "IFNULL( SUM( sum_run_duration ), 0 ) AS sum_run_duration")
                .lambda()
                .eq(LineDailyElectricityTrendEnt::getSystemSource, input.getSystemSource())
                .in(LineDailyElectricityTrendEnt::getStationName, input.getStationNameList())
        );
        return XSingleResult.success(XCopyUtils.copyNewObject(ent, GetLineDailyElectricityTrendOutput.class));
    }

    /**
     * 查询谷电占比
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetValleyElectricRateOutput }>
     */
    @Override
    public XListResult<GetValleyElectricRateOutput> queryValleyElectricRate(XContext context, GetLineDailyElectricityTrendInput input) {
        Date today = input.getStartTime();
        LineDailyElectricityTrendViewMapper mapper = context.getBean(LineDailyElectricityTrendViewMapper.class);
        List<LineDailyElectricityTrendView> list = mapper.selectValleyElectricRate(LineDailyElectricityTrendView.builder()
                .createDate(today)
                .startTime(DateUtil.offsetDay(today, -1))
                .endTime(DateUtil.offset(today, DateField.YEAR, -1))
                .stationNameList(input.getStationNameList())
                .systemSource(input.getSystemSource())
                .build());
        return XListResult.success(XCopyUtils.copyNewList(list, GetValleyElectricRateOutput.class));
    }

    /**
     * 井口分析分页
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult }<{@link GetWellDailyElectricityTrendOutput }>
     */
    @Override
    public XPageResult<GetWellDailyElectricityTrendOutput> queryWellheadAnalysisPage(XContext context, GetWellDailyElectricityTrendInput input) {
        DateTime date = DateUtil.date(input.getCreateDate());
        PageMethod.startPage(input.getPage(), input.getLimit());
        WellDailyElectricityTrendViewMapper mapper = context.getBean(WellDailyElectricityTrendViewMapper.class);
        List<WellDailyElectricityTrendView> list = mapper.selectWellheadAnalysisPage(
                WellDailyElectricityTrendView.builder()
                        .createDate(date)
                        .month(date.monthBaseOne())
                        .year(date.year())
                        .wellNumberList(input.getWellNumberList())
                        .wellNumber(input.getWellNumber())
                        .build()
        );
        PageInfo<WellDailyElectricityTrendView> pageInfo = new PageInfo<>(list);
        List<GetWellDailyElectricityTrendOutput> output;
        if (CollUtil.isNotEmpty(list)) {
            output = new ArrayList<>(list.size());
            for (WellDailyElectricityTrendView view : list) {
                output.add(GetWellDailyElectricityTrendOutput.builder()
                        .wellNumber(view.getWellNumber())
                        .spaceRunDuration(view.getSpaceRunDuration())
                        .dailyLiquidProduction(view.getDailyLiquidProduction())
                        .greenElectricityRate(view.getGreenElectricityRate())
                        .dailyCarbonReduction(view.getDailyCarbonReduction())
                        .valleyElectricRate(view.getValleyElectricRate())
                        .monthGreenElectricityRate(BaseUtils.getRate(view.getMonthDailyGreenOpenHour(), view.getMonthSpaceRunDuration()))
                        .monthDailyCarbonReduction(view.getMonthDailyGreenConsumption().multiply(BusinessConstant.BIG_DECIMAL_6_67))
                        .monthValleyElectricRate(BaseUtils.getRate(view.getMonthValleyElectricOpenHour(), view.getMonthSpaceRunDuration()))
                        .yearGreenElectricityRate(BaseUtils.getRate(view.getYearDailyGreenOpenHour(), view.getYearSpaceRunDuration()))
                        .yearDailyCarbonReduction(view.getYearDailyGreenConsumption().multiply(BusinessConstant.BIG_DECIMAL_6_67))
                        .yearValleyElectricRate(BaseUtils.getRate(view.getYearValleyElectricOpenHour(), view.getYearSpaceRunDuration()))
                        .build());
            }
        } else {
            output = Collections.emptyList();
        }
        return XPageResult.success(output, input, pageInfo.getTotal());
    }

    /**
     * 查询井日平均值
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetWellDailyElectricityTrendOutput }>
     */
    @Override
    public XSingleResult<GetWellDailyElectricityTrendOutput> queryWellDailyAverage(XContext context, GetWellDailyAverageInput input) {
        WellDailyElectricityTrendMapper mapper = context.getBean(WellDailyElectricityTrendMapper.class);
        WellDailyElectricityTrendEnt ent = mapper.selectOne(new QueryWrapper<WellDailyElectricityTrendEnt>()
                .select("IFNULL( ROUND( AVG( daily_electricity_consumption ), 2 ), 0 ) AS daily_electricity_consumption",
                        "IFNULL( ROUND( AVG( daily_liquid_production ), 2 ), 0 ) AS daily_liquid_production")
                .lambda()
                .eq(WellDailyElectricityTrendEnt::getSystemSource, input.getSystemSource())
                .in(WellDailyElectricityTrendEnt::getWellNumber, input.getWellNumberList())
        );
        return XSingleResult.success(XCopyUtils.copyNewObject(ent, GetWellDailyElectricityTrendOutput.class));
    }

    /*------------------------------------- private -------------------------------------*/

    /**
     * 获取同期用电趋势信息
     *
     * @param context      上下文
     * @param createDate   创建日期
     * @param oilFieldCode 来源系统
     * @return {@link Map }<{@link String }, {@link LineDailyElectricityTrendEnt }>
     */
    private Map<String, LineDailyElectricityTrendEnt> getLastElectricityTrendMap(XContext context, Date createDate, String oilFieldCode) {
        LineDailyElectricityTrendMapper mapper = context.getBean(LineDailyElectricityTrendMapper.class);
        List<LineDailyElectricityTrendEnt> list = mapper.selectList(new LambdaQueryWrapper<LineDailyElectricityTrendEnt>()
                .eq(CharSequenceUtil.isNotBlank(oilFieldCode), LineDailyElectricityTrendEnt::getSystemSource, oilFieldCode)
                .apply("DATE( create_date ) = {0}", DateUtil.offset(createDate, DateField.YEAR, -1))
        );
        Map<String, LineDailyElectricityTrendEnt> collect;
        if (CollUtil.isEmpty(list)) {
            collect = Collections.emptyMap();
        } else {
            collect = list.stream()
                    .collect(Collectors.toMap(LineDailyElectricityTrendEnt::getStationName, Function.identity()));
        }
        return collect;
    }

    /**
     * 查询每日井场日累计数据
     *
     * @param context      上下文
     * @param createDate   创建日期
     * @param oilFieldCode 来源系统
     * @return {@link Map }<{@link String }, {@link BigDecimal }>
     */
    private Map<String, BigDecimal> getDailyUpdatePowerMap(XContext context, Date createDate, String oilFieldCode) {
        IThirdPowerCloudService service = context.getBean(IThirdPowerCloudService.class);
        XListResult<GetThirdDailyAccumulationUpdateOutput> result = service.queryDailyUpdatePower(context, GetThirdDailyAccumulationUpdateInput.builder()
                .createDate(createDate)
                .systemSource(oilFieldCode)
                .build());
        result.throwIfFail();
        Map<String, BigDecimal> powerMap;
        List<GetThirdDailyAccumulationUpdateOutput> powerList = result.getResult();
        if (CollUtil.isEmpty(powerList)) {
            powerMap = Collections.emptyMap();
        } else {
            powerMap = powerList.stream()
                    .collect(Collectors.toMap(GetThirdDailyAccumulationUpdateOutput::getStationName, GetThirdDailyAccumulationUpdateOutput::getDailyElectricityConsumption));
        }
        return powerMap;
    }

    /**
     * 查每15分钟的有功功率
     *
     * @param context      上下文
     * @param createDate   创建日期
     * @param oilFieldCode 来源系统
     * @return {@link Map }<{@link String }, {@link List }<{@link GetThirdActivePowerOutput }>>
     */
    private Map<String, List<GetThirdActivePowerOutput>> getAvgPhotovoltaicPower(XContext context, Date createDate, String oilFieldCode) {
        IThirdPowerCloudService service = context.getBean(IThirdPowerCloudService.class);
        XListResult<GetThirdActivePowerOutput> result = service.queryAvgPhotovoltaicPower(context, GetThirdActivePowerInput.builder()
                .createDate(createDate)
                .systemSource(oilFieldCode)
                .build());
        result.throwIfFail();
        Map<String, List<GetThirdActivePowerOutput>> powerMap;
        List<GetThirdActivePowerOutput> powerList = result.getResult();
        if (CollUtil.isEmpty(powerList)) {
            powerMap = Collections.emptyMap();
        } else {
            powerMap = powerList.stream()
                    .collect(Collectors.groupingBy(GetThirdActivePowerOutput::getStationName));
        }
        return powerMap;
    }

    /**
     * 每日井口生产情况
     *
     * @param context      上下文
     * @param createDate   创建日期
     * @param oilFieldCode 来源系统
     * @return {@link Map }<{@link String }, {@link GetWellheadDailyProductionSituationOutput }>
     */
    private Map<String, GetWellheadDailyProductionSituationOutput> getWellProductionList(XContext context, Date createDate, String oilFieldCode) {
        IThirdPowerCloudService service = context.getBean(IThirdPowerCloudService.class);
        XListResult<GetWellheadDailyProductionSituationOutput> result = service.queryWellProductionList(context, GetWellheadDailyProductionSituationInput.builder()
                .createDate(createDate)
                .systemSource(oilFieldCode)
                .build());
        result.throwIfFail();
        Map<String, GetWellheadDailyProductionSituationOutput> collect;
        List<GetWellheadDailyProductionSituationOutput> list = result.getResult();
        if (CollUtil.isEmpty(list)) {
            collect = Collections.emptyMap();
        } else {
            collect = list.stream()
                    .collect(Collectors.toMap(GetWellheadDailyProductionSituationOutput::getWellNumber, Function.identity()));
        }
        return collect;
    }

    /**
     * 获取优化结果
     *
     * @param context      上下文
     * @param optimizeDate 优化日期
     * @return {@link Map }<{@link String }, {@link List }<{@link SpaceOptimizeShortDurationView }>>
     */
    private Map<String, List<SpaceOptimizeShortDurationView>> getOptimizationResult(XContext context, Date optimizeDate) {
        SpaceOptimizeShortDurationViewMapper shortMapper = context.getBean(SpaceOptimizeShortDurationViewMapper.class);
        List<SpaceOptimizeShortDurationView> durationList = shortMapper.selectOptimizationResultList(SpaceOptimizeShortDurationView.builder()
                .optimizeDate(optimizeDate)
                .build());
        Map<String, List<SpaceOptimizeShortDurationView>> lineMap;
        if (CollUtil.isEmpty(durationList)) {
            lineMap = Collections.emptyMap();
        } else {
            lineMap = durationList.stream()
                    .collect(Collectors.groupingBy(SpaceOptimizeShortDurationView::getLineName));
        }
        return lineMap;
    }

    /**
     * 获取井口日用电趋势信息
     *
     * @param productionSituationOutput 生产情况输出
     * @param wellNumber                井号
     * @param runDuration               运行持续时间
     * @param dailyGreenOpenHour        每日绿色开放时间
     * @param dailyGreenConsumption     日常绿色消费
     * @param yesterday                 昨天
     * @param valleyElectricOpenHour    山谷电力开放时间
     * @param oilFieldCode              环境
     * @return {@link WellDailyElectricityTrendView }
     */
    private WellDailyElectricityTrendView getWellDailyElectricityTrendView(GetWellheadDailyProductionSituationOutput productionSituationOutput,
                                                                           String wellNumber, BigDecimal runDuration, BigDecimal dailyGreenOpenHour,
                                                                           BigDecimal dailyGreenConsumption, Date yesterday, BigDecimal valleyElectricOpenHour, String oilFieldCode) {
        WellDailyElectricityTrendView view;
        if (ObjectUtil.isNotNull(productionSituationOutput)) {
            view = WellDailyElectricityTrendView.builder()
                    .wellNumber(wellNumber)
                    .spaceRunDuration(runDuration)
                    .dailyOpenHour(productionSituationOutput.getProductionTime())
                    .dailyGreenOpenHour(dailyGreenOpenHour)
                    .dailyGreenConsumption(dailyGreenConsumption)
                    .dailyElectricityConsumption(productionSituationOutput.getDailyElectricityConsumption())
                    .dailyLiquidProduction(productionSituationOutput.getDailyLiquidProduction())
                    //日减碳量:日绿电用电量*6.67
                    .dailyCarbonReduction(dailyGreenConsumption.multiply(BusinessConstant.BIG_DECIMAL_6_67))
                    //绿电占比:绿电开井时长/开井时长*100%
                    .greenElectricityRate(BaseUtils.getRate(dailyGreenOpenHour, runDuration))
                    .valleyElectricOpenHour(valleyElectricOpenHour)
                    //谷电占比:所有井的谷电时间开井时长/所有井开井时长保留2位小数
                    .valleyElectricRate(BaseUtils.getRate(valleyElectricOpenHour, runDuration))
                    .createDate(yesterday)
                    .systemSource(oilFieldCode)
                    .build();
        } else {
            view = WellDailyElectricityTrendView.builder()
                    .wellNumber(wellNumber)
                    .spaceRunDuration(runDuration)
                    .dailyOpenHour(BigDecimal.ZERO)
                    .dailyGreenOpenHour(dailyGreenOpenHour)
                    .dailyGreenConsumption(dailyGreenConsumption)
                    .dailyElectricityConsumption(BigDecimal.ZERO)
                    .dailyLiquidProduction(BigDecimal.ZERO)
                    //日减碳量:日绿电用电量*6.67
                    .dailyCarbonReduction(dailyGreenConsumption.multiply(BusinessConstant.BIG_DECIMAL_6_67))
                    //绿电占比:绿电开井时长/开井时长*100%
                    .greenElectricityRate(BaseUtils.getRate(dailyGreenOpenHour, runDuration))
                    .valleyElectricOpenHour(valleyElectricOpenHour)
                    //谷电占比:所有井的谷电时间开井时长/所有井开井时长保留2位小数
                    .valleyElectricRate(BaseUtils.getRate(valleyElectricOpenHour, runDuration))
                    .createDate(yesterday)
                    .systemSource(oilFieldCode)
                    .build();
        }
        return view;
    }

    /**
     * 获取线路日电量趋势
     *
     * @param ent                         ent
     * @param dailyElectricityConsumption 日用电量
     * @param photovoltaicPower           光伏发电
     * @param stationName                 车站名称
     * @param yesterday                   昨天
     * @param sumRunDuration              总运行时间
     * @param sumValleyElectricOpenHour   sum valley电力开放时间
     * @param oilFieldCode                环境
     * @return {@link LineDailyElectricityTrendView }
     */
    private LineDailyElectricityTrendView getLineDailyElectricityTrendView(LineDailyElectricityTrendEnt ent, BigDecimal dailyElectricityConsumption,
                                                                           BigDecimal photovoltaicPower, String stationName, Date yesterday, BigDecimal sumRunDuration,
                                                                           BigDecimal sumValleyElectricOpenHour, String oilFieldCode) {
        LineDailyElectricityTrendView view;
        BigDecimal municipalPower = dailyElectricityConsumption.subtract(photovoltaicPower);
        if (municipalPower.compareTo(BigDecimal.ZERO) < 0) {
            municipalPower = BigDecimal.ZERO;
        }
        if (ObjectUtil.isNotNull(ent)) {
            view = LineDailyElectricityTrendView.builder()
                    .stationName(stationName)
                    .totalPower(dailyElectricityConsumption)
                    .photovoltaicPower(photovoltaicPower)
                    //市电日用电量:总用电-光伏用电
                    .municipalPower(municipalPower)
                    .lastTotalPower(ent.getTotalPower())
                    .lastPhotovoltaicPower(ent.getPhotovoltaicPower())
                    .lastMunicipalPower(ent.getMunicipalPower())
                    //同比:本期/上期*100
                    .totalRatio(BaseUtils.getRate(dailyElectricityConsumption, ent.getTotalPower()))
                    .photovoltaicRatio(BaseUtils.getRate(photovoltaicPower, ent.getPhotovoltaicPower()))
                    .municipalRatio(BaseUtils.getRate(municipalPower, ent.getMunicipalPower()))
                    .valleyElectricOpenHour(sumValleyElectricOpenHour)
                    //谷电占比:所有井的谷电时间开井时长/所有井开井时长保留2位小数
                    .valleyElectricRate(BaseUtils.getRate(sumValleyElectricOpenHour, sumRunDuration))
                    .sumRunDuration(sumRunDuration)
                    .createDate(yesterday)
                    .systemSource(oilFieldCode)
                    .build();
        } else {
            view = LineDailyElectricityTrendView.builder()
                    .stationName(stationName)
                    .totalPower(dailyElectricityConsumption)
                    .photovoltaicPower(photovoltaicPower)
                    //市电日用电量:总用电-光伏用电
                    .municipalPower(municipalPower)
                    .lastTotalPower(BigDecimal.ZERO)
                    .lastPhotovoltaicPower(BigDecimal.ZERO)
                    .lastMunicipalPower(BigDecimal.ZERO)
                    .totalRatio(BigDecimal.ZERO)
                    .photovoltaicRatio(BigDecimal.ZERO)
                    .municipalRatio(BigDecimal.ZERO)
                    .valleyElectricOpenHour(sumValleyElectricOpenHour)
                    //谷电占比:所有井的谷电时间开井时长/所有井开井时长保留2位小数
                    .valleyElectricRate(BaseUtils.getRate(sumValleyElectricOpenHour, sumRunDuration))
                    .sumRunDuration(sumRunDuration)
                    .createDate(yesterday)
                    .systemSource(oilFieldCode)
                    .build();
        }
        return view;
    }
}