package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.BasePhotovoltaicPlantCloudService;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudInput;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudOutput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.prediction.service.IThirdDataAccessCloudService;
import pps.core.prediction.constant.ThirdPartyApiConstant;
import pps.core.prediction.entity.*;
import pps.core.prediction.mapper.*;
import pps.core.prediction.service.data.third_active_power.GetThirdActivePowerViewOutput;
import pps.core.prediction.service.data.third_active_power.ThirdActivePowerExcelData;
import pps.core.prediction.service.data.third_current_well_condition.GetThirdCurrentWellConditionViewOutput;
import pps.core.prediction.service.data.third_daily_accumulation.GetCumulativePowerGenerationOutput;
import pps.core.prediction.service.data.third_photovoltaic_power.GetThirdPhotovoltaicPowerViewOutput;
import pps.core.prediction.service.data.third_well_avg_active_power.GetThirdWellAvgActivePowerOutput;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.helper.XTransactionHelper;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 第三方数据接入Cloud模块
 *
 * @author ZWT
 * @date 2024/03/19 10:31
 */
@XText("第三方数据接入Cloud模块")
@XService
public class ThirdDataAccessCloudServiceImpl implements IThirdDataAccessCloudService {

    /**
     * 第三方有功功率定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult thirdActivePowerJob(XContext context) {
        String resultStr = ServiceUtil.doPostFormCq(context,
                ThirdPartyApiConstant.CQ_WELL_STATION_PV_DATA,
                new HashMap<>(0));
        context.getLogger().info("*** CQ_WELL_STATION_PV_DATA : {}", resultStr);
        List<GetThirdActivePowerViewOutput> outputs = JSON.parseArray(resultStr, GetThirdActivePowerViewOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            return XServiceResult.OK;
        }
        Map<String, String> collect = this.getPhotovoltaicPlantMap(context);
        List<ThirdActivePowerView> dtoList = new ArrayList<>(outputs.size());
        List<ThirdActivePowerDailyUpdateView> dtoDailyList = new ArrayList<>(outputs.size());
        DateTime now = DateTime.now();
        DateTime saveDate = DateUtil.beginOfDay(now);
        DateTime saveHour = DateUtil.beginOfHour(now);
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        LambdaQueryWrapper<ThirdActivePowerEnt> queryWrapper = new LambdaQueryWrapper<ThirdActivePowerEnt>()
                .select(ThirdActivePowerEnt::getId);
        for (GetThirdActivePowerViewOutput o : outputs) {
            BigDecimal photovoltaicPower = Optional.ofNullable(o.getPhotovoltaicPower())
                    .orElse(BigDecimal.ZERO)
                    .abs();
            BigDecimal meterPower = Optional.ofNullable(o.getMeterPower())
                    .orElse(BigDecimal.ZERO)
                    .abs();
            dtoList.add(
                    ThirdActivePowerView.builder()
                            .stationName(o.getStationName())
                            .stationId(collect.get(o.getStationName()))
                            .createDate(o.getCreateDate())
                            .photovoltaicPower(photovoltaicPower)
                            .meterPower(meterPower)
                            .systemSource(oilFieldCode)
                            .inputTime(now)
                            .build()
            );
            queryWrapper.or(w -> w.eq(ThirdActivePowerEnt::getStationName, o.getStationName())
                    .eq(ThirdActivePowerEnt::getCreateDate, o.getCreateDate())
                    .eq(ThirdActivePowerEnt::getPhotovoltaicPower, photovoltaicPower)
                    .eq(ThirdActivePowerEnt::getMeterPower, meterPower)
                    .eq(ThirdActivePowerEnt::getSystemSource, oilFieldCode)
            );
            dtoDailyList.add(
                    ThirdActivePowerDailyUpdateView.builder()
                            .stationName(o.getStationName())
                            .stationId(collect.get(o.getStationName()))
                            .createDate(o.getCreateDate())
                            .photovoltaicPower(photovoltaicPower)
                            .meterPower(meterPower)
                            .systemSource(oilFieldCode)
                            .saveDate(saveDate)
                            .saveHour(saveHour)
                            .build()
            );
        }
        return XTransactionHelper.begin(context, () -> {
            //删除重复数据
            ThirdActivePowerMapper powerMapper = context.getBean(ThirdActivePowerMapper.class);
            powerMapper.delete(queryWrapper);
            ThirdActivePowerViewMapper mapper = context.getBean(ThirdActivePowerViewMapper.class);
            //保存历史数据
            mapper.batchInsertList(dtoList);
            ThirdActivePowerDailyUpdateViewMapper dailyMapper = context.getBean(ThirdActivePowerDailyUpdateViewMapper.class);
            //删除当日最新数据
            dailyMapper.deleteByCurrentDayData(dtoDailyList, saveHour);
            //保存当日最新数据
            dailyMapper.batchInsertList(dtoDailyList);
            return XServiceResult.OK;
        });
    }

    /**
     * 第三方当日油井状态定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult thirdCurrentWellConditionJob(XContext context) {
        String resultStr = ServiceUtil.doPostFormCq(context,
                ThirdPartyApiConstant.CQ_WELL_REAL_PV_DATA,
                new HashMap<>(0));
        context.getLogger().info("*** CQ_WELL_REAL_PV_DATA : {}", resultStr);
        List<GetThirdCurrentWellConditionViewOutput> outputs = JSON.parseArray(resultStr, GetThirdCurrentWellConditionViewOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            return XServiceResult.OK;
        }
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        DateTime date = DateUtil.date();
        //先删后增
        BigDecimal cumulativeProduction;
        List<ThirdCurrentWellConditionView> dtoList = new ArrayList<>(outputs.size());
        for (GetThirdCurrentWellConditionViewOutput o : outputs) {
            cumulativeProduction = Optional.ofNullable(o.getCumulativeProduction()).orElse(BigDecimal.ZERO);
            cumulativeProduction = cumulativeProduction.compareTo(BigDecimal.ZERO) < 0 ? BigDecimal.ZERO : cumulativeProduction;
            dtoList.add(
                    ThirdCurrentWellConditionView.builder()
                            .systemSource(oilFieldCode)
                            .wellNumber(o.getWellNumber())
                            .nextTime(o.getNextTime())
                            .wellStatus(Optional.ofNullable(o.getWellStatus()).orElse("停井"))
                            .cumulativeProduction(cumulativeProduction)
                            .runningTime(o.getRunningTime())
                            .updateTime(date)
                            .build()
            );
        }
        return XTransactionHelper.begin(context, () -> {
            ThirdCurrentWellConditionViewMapper mapper = context.getBean(ThirdCurrentWellConditionViewMapper.class);
            mapper.deleteByParams(dtoList);
            mapper.batchInsertList(dtoList);
            return XServiceResult.OK;
        });
    }

    /**
     * 第三方光伏日发电量定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult thirdPhotovoltaicPowerJob(XContext context) {
        Map<String, Object> param = new HashMap<>(4);
        param.put("Calc_Date", DateUtil.yesterday().toString("yyyy-MM-dd"));
        String resultStr = ServiceUtil.doPostFormCq(context,
                ThirdPartyApiConstant.CQ_STATION_DAILY_PV_DATA,
                param);
        context.getLogger().info("*** CQ_STATION_DAILY_PV_DATA : {}", resultStr);
        List<GetThirdPhotovoltaicPowerViewOutput> outputs = JSON.parseArray(resultStr, GetThirdPhotovoltaicPowerViewOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            return XServiceResult.OK;
        }
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        BigDecimal photovoltaicPower;
        List<ThirdPhotovoltaicPowerView> dtoList = new ArrayList<>(outputs.size());
        for (GetThirdPhotovoltaicPowerViewOutput output : outputs) {
            photovoltaicPower = Optional.ofNullable(output.getPhotovoltaicPower()).orElse(BigDecimal.ZERO);
            photovoltaicPower = photovoltaicPower.compareTo(BigDecimal.ZERO) < 0 ? BigDecimal.ZERO : photovoltaicPower;
            dtoList.add(
                    ThirdPhotovoltaicPowerView.builder()
                            .stationName(output.getStationName())
                            .createDate(output.getCreateDate())
                            .photovoltaicPower(photovoltaicPower)
                            .systemSource(oilFieldCode)
                            .build()
            );
        }
        return XTransactionHelper.begin(context, () -> {
            ThirdPhotovoltaicPowerViewMapper mapper = context.getBean(ThirdPhotovoltaicPowerViewMapper.class);
            mapper.batchInsertList(dtoList);
            return XServiceResult.OK;
        });
    }

    /**
     * 能耗分析定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult energyConsumptionAnalysisJob(XContext context) {
        DateTime yesterday = DateUtil.beginOfDay(DateUtil.yesterday());
        context.getLogger().info("------ EnergyConsumptionAnalysisJob yesterday:{}", yesterday.toString());
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        //查昨日光伏发电量
        ThirdPhotovoltaicPowerMapper powerMapper = context.getBean(ThirdPhotovoltaicPowerMapper.class);
        List<ThirdPhotovoltaicPowerEnt> powerList = powerMapper.selectList(new LambdaQueryWrapper<ThirdPhotovoltaicPowerEnt>()
                .eq(ThirdPhotovoltaicPowerEnt::getCreateDate, yesterday)
                .eq(ThirdPhotovoltaicPowerEnt::getSystemSource, oilFieldCode)
        );
        if (CollUtil.isEmpty(powerList)) {
            context.getLogger().info("------ EnergyConsumptionAnalysisJob powerList isEmpty");
            return XServiceResult.OK;
        }
        //通过场站名查线路ID
        Map<String, String> lineMap = this.getBasePowerLineListByNames(context, powerList.stream()
                        .map(ThirdPhotovoltaicPowerEnt::getStationName)
                        .collect(Collectors.toList())).stream()
                .collect(Collectors.toMap(DynamicQueryBasePowerLineOutput::getLineName, DynamicQueryBasePowerLineOutput::getId));
        //查昨日井口耗电量
        WellheadDailyProductionSituationViewMapper mapper = context.getBean(WellheadDailyProductionSituationViewMapper.class);
        List<WellheadDailyProductionSituationView> situationList = mapper.selectSumDailyElectricityConsumption(yesterday, oilFieldCode);
        Map<String, BigDecimal> collect = situationList.stream()
                .collect(Collectors.toMap(WellheadDailyProductionSituationView::getLineId, WellheadDailyProductionSituationView::getDailyElectricityConsumption));
        //封装
        List<EnergyConsumptionAnalysisView> analysisList = new ArrayList<>(powerList.size());
        for (ThirdPhotovoltaicPowerEnt power : powerList) {
            String lineId = lineMap.get(power.getStationName());
            analysisList.add(EnergyConsumptionAnalysisView.builder()
                    .lineId(lineId)
                    .stationName(power.getStationName())
                    .powerGeneration(power.getPhotovoltaicPower())
                    .powerConsumption(collect.get(lineId))
                    .dataDate(power.getCreateDate())
                    .systemSource(oilFieldCode)
                    .build());
        }
        return XTransactionHelper.begin(context, () -> {
            EnergyConsumptionAnalysisViewMapper energyMapper = context.getBean(EnergyConsumptionAnalysisViewMapper.class);
            energyMapper.batchInsertList(analysisList);
            return XServiceResult.OK;
        });
    }

    /**
     * 第三方井场日累计数据定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult thirdDailyAccumulationJob(XContext context) {
        String resultStr = ServiceUtil.doPostFormCq(context,
                ThirdPartyApiConstant.CQ_GROUP_REAL_PV_DATA,
                new HashMap<>(0));
        context.getLogger().info("*** CQ_GROUP_REAL_PV_DATA : {}", resultStr);
        List<GetCumulativePowerGenerationOutput> outputs = JSON.parseArray(resultStr, GetCumulativePowerGenerationOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            return XServiceResult.OK;
        }
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        //取电站
        Map<String, String> plantMap = this.getPhotovoltaicPlantMap(context);
        DateTime now = DateUtil.date();
        DateTime beginOfDay = DateUtil.beginOfDay(now);
        //封装数据
        List<ThirdDailyAccumulationView> dtoList = new ArrayList<>(outputs.size());
        List<ThirdDailyAccumulationUpdateView> updateDtoList = new ArrayList<>(outputs.size());
        LambdaQueryWrapper<ThirdDailyAccumulationEnt> queryWrapper = new LambdaQueryWrapper<>();
        BigDecimal photovoltaicPower;
        BigDecimal dailyElectricityConsumption;
        BigDecimal dailyLiquidProduction;
        BigDecimal inPlaceConsumption;
        BigDecimal energyStorageDischarge;
        List<String> stationNameList = new ArrayList<>(outputs.size());
        for (GetCumulativePowerGenerationOutput o : outputs) {
            //数据处理
            photovoltaicPower = Optional.ofNullable(o.getPhotovoltaicPower()).orElse(BigDecimal.ZERO);
            dailyElectricityConsumption = Optional.ofNullable(o.getDailyElectricityConsumption()).orElse(BigDecimal.ZERO);
            dailyLiquidProduction = Optional.ofNullable(o.getDailyLiquidProduction()).orElse(BigDecimal.ZERO);
            inPlaceConsumption = Optional.ofNullable(o.getInPlaceConsumption()).orElse(BigDecimal.ZERO);
            energyStorageDischarge = Optional.ofNullable(o.getEnergyStorageDischarge()).orElse(BigDecimal.ZERO);
            photovoltaicPower = photovoltaicPower.compareTo(BigDecimal.ZERO) < 0 ? BigDecimal.ZERO : photovoltaicPower;
            dailyElectricityConsumption = dailyElectricityConsumption.compareTo(BigDecimal.ZERO) < 0 ? BigDecimal.ZERO : dailyElectricityConsumption;
            dailyLiquidProduction = dailyLiquidProduction.compareTo(BigDecimal.ZERO) < 0 ? BigDecimal.ZERO : dailyLiquidProduction;
            inPlaceConsumption = inPlaceConsumption.compareTo(BigDecimal.ZERO) < 0 ? BigDecimal.ZERO : inPlaceConsumption;
            energyStorageDischarge = energyStorageDischarge.compareTo(BigDecimal.ZERO) < 0 ? BigDecimal.ZERO : energyStorageDischarge;
            o.setPhotovoltaicPower(photovoltaicPower);
            o.setDailyElectricityConsumption(dailyElectricityConsumption);
            o.setDailyLiquidProduction(dailyLiquidProduction);
            o.setInPlaceConsumption(inPlaceConsumption);
            o.setEnergyStorageDischarge(energyStorageDischarge);
            //历史数据
            dtoList.add(
                    ThirdDailyAccumulationView.builder()
                            .saveDate(beginOfDay)
                            .stationId(plantMap.get(o.getStationName()))
                            .stationName(o.getStationName())
                            .createDate(now)
                            .photovoltaicPower(o.getPhotovoltaicPower())
                            .dailyElectricityConsumption(o.getDailyElectricityConsumption())
                            .dailyLiquidProduction(o.getDailyLiquidProduction())
                            .inPlaceConsumption(o.getInPlaceConsumption())
                            .energyStorageDischarge(o.getEnergyStorageDischarge())
                            .systemSource(oilFieldCode)
                            .build()
            );
            //组装去重删除条件
            queryWrapper.or(w -> w.eq(ThirdDailyAccumulationEnt::getStationName, o.getStationName())
                    .eq(ThirdDailyAccumulationEnt::getSaveDate, beginOfDay)
                    .eq(ThirdDailyAccumulationEnt::getPhotovoltaicPower, o.getPhotovoltaicPower())
                    .eq(ThirdDailyAccumulationEnt::getDailyElectricityConsumption, o.getDailyElectricityConsumption())
                    .eq(ThirdDailyAccumulationEnt::getDailyLiquidProduction, o.getDailyLiquidProduction())
                    .eq(ThirdDailyAccumulationEnt::getInPlaceConsumption, o.getInPlaceConsumption())
                    .eq(ThirdDailyAccumulationEnt::getSystemSource, oilFieldCode)
            );
            //每日最新数据
            updateDtoList.add(
                    ThirdDailyAccumulationUpdateView.builder()
                            .saveDate(beginOfDay)
                            .stationId(plantMap.get(o.getStationName()))
                            .stationName(o.getStationName())
                            .createDate(now)
                            .photovoltaicPower(Optional.ofNullable(o.getPhotovoltaicPower()).orElse(BigDecimal.ZERO))
                            .dailyElectricityConsumption(Optional.ofNullable(o.getDailyElectricityConsumption()).orElse(BigDecimal.ZERO))
                            .dailyLiquidProduction(Optional.ofNullable(o.getDailyLiquidProduction()).orElse(BigDecimal.ZERO))
                            .inPlaceConsumption(Optional.ofNullable(o.getInPlaceConsumption()).orElse(BigDecimal.ZERO))
                            .energyStorageDischarge(Optional.ofNullable(o.getEnergyStorageDischarge()).orElse(BigDecimal.ZERO))
                            .systemSource(oilFieldCode)
                            .saveHour(now)
                            .build()
            );
            //井组列表
            stationNameList.add(o.getStationName());
        }
        return XTransactionHelper.begin(context, () -> {
            //先去重
            ThirdDailyAccumulationMapper bean = context.getBean(ThirdDailyAccumulationMapper.class);
            bean.delete(queryWrapper);
            //保存当日最新数据
            ThirdDailyAccumulationViewMapper mapper = context.getBean(ThirdDailyAccumulationViewMapper.class);
            mapper.batchInsertList(dtoList);
            //删除每日最新数据
            ThirdDailyAccumulationUpdateMapper updateMapper = context.getBean(ThirdDailyAccumulationUpdateMapper.class);
            updateMapper.delete(new LambdaQueryWrapper<ThirdDailyAccumulationUpdateEnt>()
                    .eq(ThirdDailyAccumulationUpdateEnt::getSaveDate, beginOfDay)
                    .in(ThirdDailyAccumulationUpdateEnt::getStationName, stationNameList)
            );
            //保存每日最新数据
            ThirdDailyAccumulationUpdateViewMapper updateViewMapper = context.getBean(ThirdDailyAccumulationUpdateViewMapper.class);
            updateViewMapper.batchInsertList(updateDtoList);
            return XServiceResult.OK;
        });
    }

    /**
     * 第三井平均有功功率作业
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult thirdWellAvgActivePowerJob(XContext context) {
        String resultStr = ServiceUtil.doPostFormCq(context,
                ThirdPartyApiConstant.CQ_AVG_ACTIVE_POWER,
                new HashMap<>(0));
        context.getLogger().info("*** CQ_AVG_ACTIVE_POWER : {}", resultStr);
        List<GetThirdWellAvgActivePowerOutput> outputs = JSON.parseArray(resultStr, GetThirdWellAvgActivePowerOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            return XServiceResult.OK;
        }
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        DateTime beginOfDay = DateUtil.date();
        //封装数据
        List<ThirdWellAvgActivePowerView> dtoList = new ArrayList<>(outputs.size());
        LambdaQueryWrapper<ThirdWellAvgActivePowerEnt> queryWrapper = new LambdaQueryWrapper<>();
        BigDecimal avgActivePower;
        for (GetThirdWellAvgActivePowerOutput o : outputs) {
            avgActivePower = Optional.ofNullable(o.getAvgActivePower()).orElse(BigDecimal.ZERO);
            avgActivePower = avgActivePower.compareTo(BigDecimal.ZERO) < 0 ? BigDecimal.ZERO : avgActivePower;
            o.setAvgActivePower(avgActivePower);
            dtoList.add(
                    ThirdWellAvgActivePowerView.builder()
                            .wellNumber(o.getWellNumber())
                            .inputTime(o.getInputTime())
                            .avgActivePower(o.getAvgActivePower())
                            .systemSource(oilFieldCode)
                            .saveDate(beginOfDay)
                            .build()
            );
            //组装删除条件
            queryWrapper.or(w -> w.eq(ThirdWellAvgActivePowerEnt::getWellNumber, o.getWellNumber())
                    .eq(ThirdWellAvgActivePowerEnt::getInputTime, o.getInputTime())
                    .eq(ThirdWellAvgActivePowerEnt::getAvgActivePower, o.getAvgActivePower())
                    .eq(ThirdWellAvgActivePowerEnt::getSystemSource, oilFieldCode)
            );
        }
        return XTransactionHelper.begin(context, () -> {
            //删除重复数据
            ThirdWellAvgActivePowerMapper bean = context.getBean(ThirdWellAvgActivePowerMapper.class);
            bean.delete(queryWrapper);
            //保存当日最新数据
            ThirdWellAvgActivePowerViewMapper mapper = context.getBean(ThirdWellAvgActivePowerViewMapper.class);
            mapper.batchInsertList(dtoList);
            return XServiceResult.OK;
        });
    }

    /**
     * 第三有功功率历史
     *
     * @param context        上下文
     * @param cachedDataList 缓存数据列表
     * @return {@link XServiceResult}
     */
    public XServiceResult thirdActivePowerHistory(XContext context, List<ThirdActivePowerExcelData> cachedDataList) {
        Map<String, String> collect = this.getPhotovoltaicPlantMap(context);
        LambdaQueryWrapper<ThirdActivePowerEnt> queryWrapper = new LambdaQueryWrapper<ThirdActivePowerEnt>()
                .select(ThirdActivePowerEnt::getId);
        List<ThirdActivePowerView> dtoList = new ArrayList<>(cachedDataList.size());
        List<ThirdActivePowerDailyUpdateView> dtoDailyList = new ArrayList<>(cachedDataList.size());
        BigDecimal photovoltaicPower;
        BigDecimal meterPower;
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        for (ThirdActivePowerExcelData o : cachedDataList) {
            photovoltaicPower = Optional.ofNullable(o.getPhotovoltaicPower())
                    .orElse(BigDecimal.ZERO)
                    .abs();
            meterPower = Optional.ofNullable(o.getMeterPower())
                    .orElse(BigDecimal.ZERO)
                    .abs();
            dtoList.add(
                    ThirdActivePowerView.builder()
                            .stationName(o.getStationName())
                            .stationId(collect.get(o.getStationName()))
                            .createDate(o.getCreateDate())
                            .photovoltaicPower(photovoltaicPower)
                            .meterPower(meterPower)
                            .systemSource(oilFieldCode)
                            .inputTime(o.getCreateDate())
                            .build()
            );
            queryWrapper.or(w -> w.eq(ThirdActivePowerEnt::getStationName, o.getStationName())
                    .eq(ThirdActivePowerEnt::getCreateDate, o.getCreateDate())
                    .eq(ThirdActivePowerEnt::getSystemSource, oilFieldCode)
            );
            dtoDailyList.add(
                    ThirdActivePowerDailyUpdateView.builder()
                            .stationName(o.getStationName())
                            .stationId(collect.get(o.getStationName()))
                            .createDate(o.getCreateDate())
                            .photovoltaicPower(photovoltaicPower)
                            .meterPower(meterPower)
                            .systemSource(oilFieldCode)
                            .saveDate(DateUtil.beginOfDay(o.getCreateDate()))
                            .saveHour(DateUtil.beginOfHour(o.getCreateDate()))
                            .build()
            );
        }
        return XTransactionHelper.begin(context, () -> {
            //删除重复数据
            ThirdActivePowerMapper powerMapper = context.getBean(ThirdActivePowerMapper.class);
            List<ThirdActivePowerEnt> list = powerMapper.selectList(queryWrapper);
            if (CollUtil.isNotEmpty(list)) {
                powerMapper.deleteBatchIds(list.stream()
                        .map(ThirdActivePowerEnt::getId)
                        .collect(Collectors.toList()));
            }
            ThirdActivePowerViewMapper mapper = context.getBean(ThirdActivePowerViewMapper.class);
            //保存历史数据
            mapper.batchInsertList(dtoList);
            ThirdActivePowerDailyUpdateViewMapper dailyMapper = context.getBean(ThirdActivePowerDailyUpdateViewMapper.class);
            //删除当日最新数据
            dailyMapper.deleteByCurrentDayDataList(dtoDailyList, dtoDailyList.stream()
                    .map(ThirdActivePowerDailyUpdateView::getSaveHour)
                    .distinct()
                    .collect(Collectors.toList()));
            //保存当日最新数据
            dailyMapper.batchInsertList(dtoDailyList);
            return XServiceResult.OK;
        });
    }

    /*----------------------------------- private -----------------------------------*/

    /**
     * 获取光伏电站Map
     *
     * @param context 上下文
     * @return {@link Map}<{@link String}, {@link String}>
     */
    private Map<String, String> getPhotovoltaicPlantMap(XContext context) {
        XListResult<GetBasePhotovoltaicPlantCloudOutput> result = context.getBean(BasePhotovoltaicPlantCloudService.class)
                .getBasePhotovoltaicPlantList(context, new GetBasePhotovoltaicPlantCloudInput());
        return result.getResult().stream()
                .collect(Collectors.toMap(GetBasePhotovoltaicPlantCloudOutput::getStationName, GetBasePhotovoltaicPlantCloudOutput::getId));
    }

    /**
     * 查询线路列表通过线路名称
     *
     * @param context   上下文
     * @param lineNames ou-id列表
     * @return {@link List}<{@link DynamicQueryBasePowerLineOutput}>
     */
    private List<DynamicQueryBasePowerLineOutput> getBasePowerLineListByNames(XContext context, List<String> lineNames) {
        return ServiceUtil.getPowerLineList(context, DynamicQueryBasePowerLineInput.builder()
                .lineNames(lineNames)
                .build());
    }
}
